; ===============================================================
; OTCOB0.ASM  Start up code for DOS
; October 1994.
; ===============================================================

; -----------------------------------------------------
; Segment and group declarations

_TEXT       SEGMENT BYTE PUBLIC 'CODE'
_TEXT       ENDS

FAR_DATA    SEGMENT PARA PUBLIC 'FAR_DATA'
FAR_DATA    ENDS

FAR_BSS     SEGMENT PARA PUBLIC 'FAR_BSS'
FAR_BSS     ENDS

_DATA       SEGMENT PARA PUBLIC 'DATA'
_DATA       ENDS

CONST       SEGMENT PARA PUBLIC 'CONST'
CONST       ENDS

_BSS        SEGMENT PARA PUBLIC 'BSS'
_BSS        ENDS

STACK       SEGMENT      STACK  'STACK'
            DB  8192 dup(?)   ; if other value, change also when
                              ; calling int 21h ah=4ah.
STACK       ENDS

DGROUP      GROUP  _DATA

; -----------------------------------------------------
; External references
; extrn       _main:FAR

; ===============================================================
_TEXT       SEGMENT BYTE PUBLIC 'CODE'
ASSUME  CS:_TEXT

HCOB0_MAIN   PROC    NEAR

; -----------------------------------------------------
; At the start,
;    CS points to the _TEXT segment
;    DS and ES both point to the program segment prefix
;    SS points to the STACK segment, SP points to the top of the stack
;    Set ds to DGROUP

     mov     ax, _DATA
     mov     ds, ax
     ASSUME  DS:DGROUP

;    Save general infos
     call    HCOB0_SAVE_INFOS

;    Save several vectors and install default divide by zero handler
;;;; call    HCOB0_SaveVectors  ; you may wish to add it to this program

;    Count the number of environment variables
     call    HCOB0_TEST_ENV
     jc      HCOB0_MAIN_EXIT

;    Release excess of memory
     call    HCOB0_RESIZE_MEM
     jc      HCOB0_MAIN_EXIT

;    If more than 20 files
     call    HCOB0_FILE_HANDLE
     jc      HCOB0_MAIN_EXIT

;    Save time
     call    HCOB0_SAVE_TIME

;    Mov DGROUP to ES
     mov     ax, word ptr cs:[DGROUP_ADDRESS]
     mov     es, ax

;    --------------------
     call    far ptr MAIN_PROGRAM
;    --------------------

HCOB0_MAIN_EXIT:
;    Exit to DOS
     mov     bp, sp
     mov     ah, 4Ch
     mov     al, byte ptr [bp+2]  ; return value in [bp+2]
     int     21h

HCOB0_MAIN  ENDP

; ===============================================================
; 1. Save general information
; ===============================================================
HCOB0_SAVE_INFOS   PROC    NEAR

PSPHigh     equ  0002h
PSPEnv      equ  002ch
PSPCmd      equ  0080h
PSPCmdMaxLg equ  0080h
;PSPCmdMaxLg equ  000FFh - PSPCmd + 1

;    Save DGROUP segment address
     mov     word ptr cs:[DGROUP_ADDRESS], ax

;    Save Program Segment Prefix address
     mov     ax, es
     mov     word ptr [_PSP], ax

;    Save Environment address
     mov     ax, word ptr es:[PSPEnv]
     mov     word ptr [_ENVSEG], ax

;    Save highest memory segment address
     mov     ax, word ptr es:[PSPHigh]
     mov     word ptr [_HEAPTOP_SEG], ax

;    Save the command tail
     mov     cx,word ptr es:[PSPCmd]    ; get the length of the command tail
     mov     ch, 0
     cmp     cl, ch
     je      end_copy_command_tail ; jump if = 0
     cmp     cx, PSPCmdMaxLg
     ja      end_copy_command_tail ; jump if > 127
     mov     word ptr [_COMMAND_TAIL_LG], cx
     mov     bx, es             ; xchg ds, es
     mov     dx, ds
     mov     es, dx             ; now es=_DATA
     mov     ds, bx             ; and ds=PSP
     mov     si, PSPCmd+1       ; copy PSPCmd+1 to _COMMAND_TAIL
     mov     di, OFFSET _COMMAND_TAIL
     cld
     rep     movsb
     mov     es, bx             ; xchg ds, es
     mov     ds, dx             ; now es=PSP and ds=_DATA
end_copy_command_tail:

;    Save DOS version number
     mov     ah, 30h
     int     21h                ; get DOS version number
     mov     word ptr[_DOS_VERSION], ax ; Keep major and minor version number

     ret

HCOB0_SAVE_INFOS   ENDP


; ===============================================================
; 2. Count the number of environment variables and compute the size.
;    Each variable is ended by a 0 and a zero-length variable stops
;    the environment. The environment can NOT be greater than 32k.
; ===============================================================
HCOB0_TEST_ENV   PROC   NEAR

     push    es

     mov     ax, word ptr [_ENVSEG]
     mov     es, ax          ; es = _envseg
     xor     ax, ax
     mov     bx, ax
     mov     di, ax          ; ax = bx = di = 0
     cld
     mov     cx, 07FFFh      ; Environment cannot be > 32 Kbytes

Test_Env_Loop:
     repnz   scasb           ; Loop until es:[di] = 0
     jcxz    Test_Env_KO     ; Bad environment !!!

     inc     bx              ; BX = Nb environment variables
     cmp     byte ptr es:[di], al     ; es:[di] = 0 ?
     jne     Test_Env_Loop   ; Next variable ...

     or      ch, 80h
     neg     cx
     mov     word ptr [_ENV_LG], cx     ; Save Environment size
;    mov     cx, dPtrSize / 2
;    shl     bx, cl
;    add     bx, dPtrSize * 4
;    and     bx, not ((dPtrSize * 4) - 1)
     mov     word ptr [_ENV_SIZE], bx    ; Save Environment Variables Nb.
     jmp     Test_Env_OK

Test_Env_KO:
     mov     ah, 9       ; Display error message (if it is possible)
     mov     dx, OFFSET ErrorEnvTest
     int     21h
     stc
     jmp End_Test_Env

Test_Env_OK:
     clc

End_Test_Env:
     pop     es

     ret

HCOB0_TEST_ENV   ENDP

; ===============================================================
; 3. Release excess of memory.
; ===============================================================
HCOB0_RESIZE_MEM   PROC   NEAR

     mov     ax, es        ; let ax = segment of PSP base
     mov     bx, ss        ; let bx = segment of stack base
     sub     bx, ax        ; reserve seg stack - seg psp
     add     bx, 512       ; plus paragraphs of stack (=8192/16)
     inc     bx            ; round up
     mov     ah, 4ah       ; fxn 4ah = resize memory block
     int     21h

     ret

HCOB0_RESIZE_MEM   ENDP

; ===============================================================
; 4. If default number of file handles have changed then tell DOS
; ===============================================================
HCOB0_FILE_HANDLE   PROC   NEAR

     cmp     word ptr [__nfile], 20
     jbe     HFile_OK

     mov     ax, _DOS_VERSION
     cmp     ah, 3         ; Check for >= DOS 3.3
     jb      HFile_KO
     ja      Change_File_Handle
     cmp     al, 1Eh
     jb      HFile_KO

Change_File_Handle:
     mov     ax, 5801h      ; Set last fit allocation
     mov     bx, 2
     int     21h
     jc      HFile_KO

     mov     ah, 67h        ; Expand handle table
     mov     bx, word ptr [__nfile]
     int     21h
     jc      HFile_KO

     mov     ah, 48h        ; Allocate 16 bytes to find new
     mov     bx, 1          ;   top of memory address
     int     21h
     jc      HFile_KO
     inc     ax             ; Adjust address to point after block
     mov     word ptr [_HEAPTOP_SEG], ax

     dec     ax             ; Change back and release block
     mov     es, ax
     mov     ah, 49h
     int     21h
     jc      HFile_KO

     mov     ax, 5801h      ; Set first fit allocation
     mov     bx, 0
     int     21h
     jnc     HFile_OK

HFile_KO:
     mov     ah, 9       ; Display error message (if it is possible)
     mov     dx, OFFSET ErrorHFile
     int     21h
     stc
     jmp     End_File_Handle

HFile_OK:
     clc

End_File_Handle:
     ret

HCOB0_FILE_HANDLE   ENDP

; ===============================================================
; 5. Save time
; ===============================================================
HCOB0_SAVE_TIME   PROC    NEAR

     mov     ah, 0
     int     1ah                     ; get current BIOS time in ticks
     mov     word ptr [_START_TIME],dx ; save it for clock() fn
     mov     word ptr [_START_TIME+2],cx
     or      al,al                   ; was midnight flag set?
     jz      NotMidnight
     mov     ax,40h                  ; set BIOS midnight flag
     mov     es,ax                   ;  at 40:70
     mov     bx,70h
     mov     byte ptr es:[bx],1

NotMidnight:
     ret

HCOB0_SAVE_TIME   ENDP

; ===============================================================
; The DGROUP_ADDRESS variable is used to reload DS with DGROUP

DGROUP_ADDRESS  DW 0

_TEXT           ENDS

; ===============================================================
_DATA       SEGMENT PARA PUBLIC 'DATA'

; Miscellaneous variables
;
_PSP             DW  0
                                    PUBLIC _PSP
_ENVSEG          DW  0
                                    PUBLIC _ENVSEG
_ENV_LG          DW  0
                                    PUBLIC _ENV_LG
_ENV_SIZE        DW  0
                                    PUBLIC _ENV_SIZE
_HEAPTOP         DW  0
                                    PUBLIC _HEAPTOP
_HEAPTOP_SEG     DW  0
                                    PUBLIC _HEAPTOP_SEG
_COMMAND_TAIL_LG DW  0
                                    PUBLIC _COMMAND_TAIL_LG
_COMMAND_TAIL    DB  128 DUP (0)
                                    PUBLIC _COMMAND_TAIL
_DOS_VERSION     DW  0
                                    PUBLIC _DOS_VERSION
_START_TIME      DW  0, 0
                                    PUBLIC _START_TIME
_HCOBOL_ERROR_NO       DW  0
                                    PUBLIC _HCOBOL_ERROR_NO
_HCOBOL_LINAGE_COUNTER DW  0
                                    PUBLIC _HCOBOL_LINAGE_COUNTER
_HCOBOL_DATE           DB  '010101'
                                    PUBLIC _HCOBOL_DATE
_HCOBOL_DAY            DB  '01001'
                                    PUBLIC _HCOBOL_DAY
_HCOBOL_DAY_OF_WEEK    DB  '1'
                                    PUBLIC _HCOBOL_DAY_OF_WEEK
_HCOBOL_TIME           DB  '01010101'
                                    PUBLIC _HCOBOL_TIME
_HCOBOL_TALLY          DW  0
                                    PUBLIC _HCOBOL_TALLY

;EXTRN __nfile:word
__nfile          DW  0

ErrorEnvTest     DB 'Error: bad environment',13,10,'$'
ErrorHFile       DB 'Error: problem with file handles. ',13,10,'$'

_DATA            ENDS
; ===============================================================

_TEXT       SEGMENT BYTE PUBLIC 'CODE'
ASSUME  CS:_TEXT
MAIN_PROGRAM    PROC  FAR

     nop

MAIN_PROGRAM     ENDP
_TEXT            ENDS

; ===============================================================

        END      HCOB0_MAIN       ; entry point
